function CreateFigure2a
%Create figure 2a

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%INITIALIZATION%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Create a trajectory of a single object in one dimension moving under a
%random walk of standard deviation sigma
sigma = 2;
mu = 0;
numFrames = 1;
cellBoundary = 20;
numDimensions = 1;
position{1} = CreateTrajectory(numDimensions, cellBoundary, numFrames, sigma, mu, cellBoundary/2);

%Create a set of particles that define our prior belief
sigmaLowHigh = [0 3];
muLowHigh = [0 0];
numPriorParticles = 1000;
priorParticles = rand(numPriorParticles, 2*numDimensions+1)*(sigmaLowHigh(2) - sigmaLowHigh(1)) + sigmaLowHigh(1);
priorParticles(:,(numDimensions+1):(2*numDimensions)) = rand(size(priorParticles(:,(numDimensions+1):(2*numDimensions))))*(muLowHigh(2) - muLowHigh(1)) + muLowHigh(1);
priorParticles(:,2*numDimensions+1) = 1/numPriorParticles;

%Set up the pixel structure of the image
resolution = 0.2;
for(iFrame = 1:numFrames)
    pixelBoundary{iFrame} = 0:resolution:cellBoundary;
end

%Run the acquisition and learning algorithm for 1 frame to get the particle
%set
truth{1} = [sigma mu];
numParticles = 500000;
intelligentAcquisition = InitializeIntelligentAcquisition;
[predictions, uncertainty, estUncertainty, futureUncertainty, particle] = LearnModel([1 1 1 1], position, cellBoundary, numParticles, pixelBoundary, truth, intelligentAcquisition,priorParticles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%MAIN CODE%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Now look at the reward and probability for the next frame
%Propogate the particles according to their motion models
particle{1} = MoveParticles(particle{1}, numDimensions, cellBoundary);

%Find the pixel that each particle lies in
pixel = FindPixels(pixelBoundary{1}, particle{1}, numDimensions);

%Find the number of particles in each pixel
probability = FindPixelProbability(pixel, numDimensions,-1);

%Find the reward associated with acquiring each pixel
reward = FindPixelReward(pixel, numDimensions, particle{1}, estUncertainty(end),[],[], numDimensions+1,3*numDimensions,ones(2*numDimensions,1),-1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%SHOW RESULTS%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

numPixels = length(reward);
parameters.XData{1} = 1:numPixels;
parameters.YData{1} = reward/reward(51);
parameters.colorStyle{1} = 'r';
parameters.XData{2} = 1:numPixels;
parameters.YData{2} = reward./probability;
parameters.YData{2} = parameters.YData{2}/parameters.YData{2}(51);
parameters.colorStyle{2} = 'k';
parameters.lineStyle{2} = ':';
parameters.XData{3} = 1:numPixels;
parameters.YData{3} = reward./(probability+0.01*sum(probability));
parameters.YData{3} = parameters.YData{3}/parameters.YData{3}(51);
%parameters.YData{2} = 
parameters.colorStyle{3} = 'b';
parameters.lineStyle{3} = '--';
parameters.YRange{1} = [0 1];
parameters.XLabels{1} = 'Pixel';
parameters.YLabels{1} = 'Benefit';
bimagic_plot(parameters);
