function CreateFigure4ab
%Create figures 4a and 4b

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%INITIALIZATION%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
numFrames = 100;
cellBoundary = 10;
numDimensions = 1;

%Create a set of particles that define our prior belief
sigmaLowHigh = [0 5];
muLowHigh = [-0.1 0.1];
numPriorParticles = 1000;
priorParticles = rand(numPriorParticles, 2*numDimensions+1)*(sigmaLowHigh(2) - sigmaLowHigh(1)) + sigmaLowHigh(1);
priorParticles(:,(numDimensions+1):(2*numDimensions)) = rand(size(priorParticles(:,(numDimensions+1):(2*numDimensions))))*(muLowHigh(2) - muLowHigh(1)) + muLowHigh(1);
priorParticles(:,2*numDimensions+1) = 1/numPriorParticles;

%Set up the pixel structure of the image
resolution = 0.5;
for i = 1:numFrames
    pixelBoundary{i} = 0:resolution:cellBoundary;
end

numParticles = 100000;
numTrajectories = 50;
for(i = 1:4)
    parameters.YData{i} = 0;
    captured{i} = 0;
end
numObjects = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%MAIN CODE%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Loop through several trials
for(iTrajectory = 1:numTrajectories)
    for(iObject = 1:numObjects)
        %Choose sigma and mu randomly based on the prior belief
        sigma{iObject} = rand(1,1)*(sigmaLowHigh(2) - sigmaLowHigh(1))+sigmaLowHigh(1);
        mu{iObject} = rand(1,1)*(muLowHigh(2) - muLowHigh(1))+muLowHigh(1);
        truth{iObject} = [sigma{iObject} mu{iObject}];
        
        %Create a trajectory of the given velocity and standard deviation
        position{iObject} = CreateTrajectory(length(sigma{1}), cellBoundary,numFrames,sigma{iObject},mu{iObject},-1);
    end
    iTrajectory
    intelligentAcquisition = InitializeIntelligentAcquisition();
    
    %%%First strategy: acquire all pixels
    %Learn the model of the trajectory
    [predictions{1},uncertainty,estUncertainty, futureUncertainty, particle, tcaptured] = LearnModel([1 1 1 1],position, cellBoundary, numParticles, pixelBoundary, truth, intelligentAcquisition,priorParticles);
    parameters.XData{1} = 1:numFrames;
    parameters.YData{1} = parameters.YData{1}+uncertainty;
    captured{1} = captured{1} + tcaptured;
    parameters.colorStyle{1} = 'k';
    parameters.lineStyle{1} = ':';
    
    %%%Second strategy: acquire pixels with highest benefit
    %%%This could be any proportion of pixels, depending on the ratio of time cost
    %%%to photobleaching cost
    intelligentAcquisition.whereToAcquire = 6;
    intelligentAcquisition.timeCost = 1;
    intelligentAcquisition.photobleachingCost = 1;
    intelligentAcquisition.numPixels = round(length(pixelBoundary{1})/4); %caps the number of pixels that can be acquired in a frame
    %Learn the model of the trajectory using the given acquisition strategy
    [predictions{2},uncertainty,estUncertainty, futureUncertainty, particle, tcaptured] = LearnModel([1 1 1 1],position, cellBoundary, numParticles, pixelBoundary, truth,intelligentAcquisition,priorParticles);
    parameters.XData{2} = 1:numFrames;
    parameters.YData{2} = parameters.YData{2}+uncertainty;
    captured{2} = captured{2} + tcaptured;
    parameters.colorStyle{2} = 'r';
    
    %%%Third strategy: acquire 25% of pixels most likely to contain object
    intelligentAcquisition.whereToAcquire = 2;
    intelligentAcquisition.numPixels = round(length(pixelBoundary{1})/4);
    %Learn the model of the trajectory using the given acquisition strategy
    [predictions{3},uncertainty,estUncertainty, futureUncertainty, particle, tcaptured] = LearnModel([1 1 1 1],position, cellBoundary, numParticles, pixelBoundary, truth,intelligentAcquisition,priorParticles);
    parameters.XData{3} = 1:numFrames;
    parameters.YData{3} = parameters.YData{3}+uncertainty;
    captured{3} = captured{3} + tcaptured;
    parameters.colorStyle{3} = 'b';
    parameters.lineStyle{3} = '--';
    
    %%%Fourth strategy: acquire 25% of pixels randomly
    intelligentAcquisition.whereToAcquire = 3;
    %Learn the model of the trajectory using the given acquisition strategy
    [predictions{4},uncertainty,estUncertainty, futureUncertainty, particle, tcaptured] = LearnModel([1 1 1 1],position, cellBoundary, numParticles, pixelBoundary, truth,intelligentAcquisition,priorParticles);
    parameters.XData{4} = 1:numFrames;
    parameters.YData{4} = parameters.YData{4}+uncertainty;
    captured{4} = captured{4} + tcaptured;
    parameters.colorStyle{4} = 'k';
    parameters.lineStyle{4} = '-.';
end

%Normalize such that the initial error is 1
normFactor = parameters.YData{1}(1);
parameters.YData{1} = parameters.YData{1}/normFactor;
parameters.YData{2} = parameters.YData{2}/normFactor;
parameters.YData{3} = parameters.YData{3}/normFactor;
parameters.YData{4} = parameters.YData{4}/normFactor;
parameters.YRange{1} = [0 1];

%%%Plot the results against time
bimagic_plot(parameters);

%%%Plot the results against photobleaching
parameters2 = parameters;
parameters2.XData{1} = cumsum(captured{1})/numTrajectories;
parameters2.XData{2} = cumsum(captured{2})/numTrajectories;
parameters2.XData{3} = cumsum(captured{3})/numTrajectories;
parameters2.XData{4} = cumsum(captured{4})/numTrajectories;


bimagic_plot(parameters2);