function CreateFigure6
%Create figure 6

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%INITIALIZATION%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
numFrames = 50;
cellBoundary = 10;
numDimensions = 1;

%Create a set of particles that define our prior belief
sigmaLowHigh = [0 1];
muLowHigh = [-0.1 0.1];
numPriorParticles = 1000;
priorParticles = rand(numPriorParticles, 2*numDimensions+1)*(sigmaLowHigh(2) - sigmaLowHigh(1)) + sigmaLowHigh(1);
priorParticles(:,(numDimensions+1):(2*numDimensions)) = rand(size(priorParticles(:,(numDimensions+1):(2*numDimensions))))*(muLowHigh(2) - muLowHigh(1)) + muLowHigh(1);
priorParticles(:,2*numDimensions+1) = 1/numPriorParticles;

%Set up the pixel structure of the image
resolution = 1;
for i = 1:numFrames
    pixelBoundary{i} = 0:resolution:cellBoundary;
end

numParticles = 100000;
numTrajectories = 20;

%Produce a set of trajectories
for(iTrajectory=1:numTrajectories)
        %Choose sigma and mu randomly based on the prior belief
        sigma{iTrajectory} = rand(1,1)*(sigmaLowHigh(2) - sigmaLowHigh(1))+sigmaLowHigh(1);    
        mu{iTrajectory} = rand(1,1)*(muLowHigh(2) - muLowHigh(1))+muLowHigh(1);  
        
        %Create a trajectory of the given velocity and standard
        %deviation
        position{iTrajectory} = CreateTrajectory(length(sigma{iTrajectory}), cellBoundary,numFrames,sigma{iTrajectory},mu{iTrajectory},5.5);
end

%Loop through our set of trajectories
for iTrajectory = 1:numTrajectories
    intelligentAcquisition = InitializeIntelligentAcquisition();
    intelligentAcquisition.numFramePeeks = 5;

    %Learn the model of each trajectory, recording the true uncertainty at
    %every frame, and the expected benefit of continued acquisition
    pos{1} = position{iTrajectory};
    truth{1} = [sigma{iTrajectory} mu{iTrajectory}];
    [predictions,uncertainty{iTrajectory},estUncertainty{iTrajectory}, futureUncertainty, particle, captured, output{iTrajectory}] = LearnModel([0 1 1 1],pos, cellBoundary, numParticles, pixelBoundary, truth, intelligentAcquisition, priorParticles);
end

%Test 1: Test different stopping thresholds, and record the number of
%frames and the final uncertainty for each of these thresholds

%Initialize the set of thresholds to test...
threshold = (12:450) * 0.00005

result{1} = zeros(length(threshold),2);
result{2} = zeros(numFrames,2);

%Loop through the thresholds and trajectories
for(iThreshold = 1:length(threshold))
    for(iTrajectory = 1:numTrajectories)
        added = 0;
        for(iFrame = 2:numFrames)
            %Determine whether acquisition would cease at this point
            if(max(output{iTrajectory}.stoppingThreshold(:,iFrame)) < threshold(iThreshold))
                result{1}(iThreshold,1) = result{1}(iThreshold,1) + iFrame-1;
                result{1}(iThreshold,2) = result{1}(iThreshold,2) + uncertainty{iTrajectory}(iFrame-1);
                added = 1;
                break;
            end
        end
        %If acquisition never ceased and we run out of frames, assume
        %acquisition ceases after the final frame
        if(added == 0)
            result{1}(iThreshold,1) = result{1}(iThreshold,1) + numFrames;
            result{1}(iThreshold,2) = result{1}(iThreshold,2) + uncertainty{iTrajectory}(numFrames);
        end
    end
end

%Test 2: Record the final uncertainties when we stop after a fixed number
%of frames
for(iTrajectory = 1:numTrajectories)    
    for(iFrame = 1:numFrames)
            result{2}(iFrame,1) = result{2}(iFrame,1) + iFrame;
            result{2}(iFrame,2) = result{2}(iFrame,2) + uncertainty{iTrajectory}(iFrame);    
    end
end

%Normalize results
indices = find(result{1}(:,1) ~= 0);
parameters.XData{1} = result{1}(indices,1)/numTrajectories;
parameters.YData{1} = result{1}(indices,2)/numTrajectories;
indices = find(result{2}(:,1) ~= 0);
parameters.XData{2} = result{2}(indices,1)/numTrajectories;
parameters.YData{2} = result{2}(indices,2)/numTrajectories;
parameters.YData{1}=parameters.YData{1}/parameters.YData{2}(1);
parameters.YData{2}=parameters.YData{2}/parameters.YData{2}(1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%SHOW RESULTS%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters.colorStyle{1} = 'r';
parameters.markerStyle{1} = 'x';
parameters.colorStyle{2} = 'b';
parameters.lineStyle{2} = '--';
parameters.XLabels{1} = 'Number of frames';
parameters.YLabels{1} = 'Average error';
bimagic_plot(parameters);
a=1;

