function [reward pixelProperties] = FindPixelReward(pixel, numDimensions, particle, estUncertainty, includedPixels, pixelProperties, unknownStart, unknownEnd, unknownWeights,numPixels)
%This function finds the reward associated with acquiring any pixel, with
%the assumption that the pixels in "includedPixels" are also acquired. 

%If the number of pixels in the image is unspecified, determine it here.
if(numPixels == -1)
    numPixels = max(pixel(:,numDimensions+1));
end

%If the pixel properties have not yet been calculated, calculate them now.
if(isempty(pixelProperties))
    pixelProperties = zeros(numPixels,2);
    for iPixel = 1:numPixels
        indices = find(pixel(:,numDimensions+1) == iPixel);
        pixelProperties(iPixel,1) = length(indices);
        pixelProperties(iPixel,2:(2+unknownEnd-unknownStart)) = sum(particle(indices,unknownStart:unknownEnd));
    end
end

tempPixel = pixel;
tempPixel(:,numDimensions+1) = 1;
futureUncertainty = zeros(1,numPixels);

%Find quantities relevant to reward calculation
totalSum = sum(particle(:,unknownStart:unknownEnd));
totalSumSquared = sum(particle(:,unknownStart:unknownEnd).^2)*unknownWeights;

if(isempty(includedPixels))
    includedPixelSumSquaredComponent = 0;
else
    includedPixelSumSquaredComponent = sum(sum(pixelProperties(includedPixels,2:end).^2,2)./pixelProperties(includedPixels,1));
end
includedPixelSumComponent = sum(pixelProperties(includedPixels,2:end));
includedPixelLength = sum(pixelProperties(includedPixels,1));

%Determine which pixels we need to examine
remainingPixels = 1:numPixels;
remainingPixels(includedPixels) = 0;
remainingPixels = remainingPixels(remainingPixels > 0);

%Find the expected uncertainty if we acquire the included pixel set in
%addition to a given remaining pixel
for iPixel = remainingPixels
    indices = find(pixel(:,numDimensions+1) == iPixel);
    tempPixel(:,numDimensions+1) = 1;
    tempPixel(indices,numDimensions+1) = iPixel;
    futureUncertainty(iPixel) = 2/size(particle,1)*(totalSumSquared - includedPixelSumSquaredComponent - sum(pixelProperties(iPixel,2:end).^2)/pixelProperties(iPixel,1) - sum((totalSum-includedPixelSumComponent-pixelProperties(iPixel,2:end)).^2)/(1 + size(particle,1)-includedPixelLength - pixelProperties(iPixel,1)));
end

%Subtract the expected future uncertainty from the estimated current
%uncertainty to find the reward associated with each pixel
reward = estUncertainty - futureUncertainty;

%The reward associated with acquiring a pixel already included in our set
%is zero
reward(includedPixels) = 0;