function parameters = bimagic_plot(parameters);

%% Input checking and validation, and variable initialization
parameters  = check_parameters(parameters);

global colorStyle;
global lineStyle;
global markStyle;
global usedPlotStyles;

colorStyle      = {'r', 'g', 'b', 'c', 'y', 'm', 'y', 'k', 'w'};
lineStyle       = {'-', '--', ':', '-.'};
markStyle       = {'+', 'o', '*', '.', 'x', 's', 'd', '^', 'v', '>', '<', 'p', 'h'};
usedPlotStyles  = {};

whiteColor      = [1, 1, 1];
greyColor       = [0.85, 0.85, 0.85];

figure;
figHandle       = gcf;
set(figHandle, 'Color', whiteColor);

axHandles(1)    = axes('XAxisLocation', 'top', 'YAxisLocation', 'right');

if(~isempty(parameters.XRange{2}))
    set(axHandles(1), 'XLim', parameters.XRange{2});
else
    set(axHandles(1), 'XLim', parameters.XRange{1});
end

if(strcmp(parameters.XTick{2}, 'auto'))
    set(axHandles(1), 'XTickLabelMode', 'auto');
else
    set(axHandles(1), 'XTick', parameters.XTick{2});
    set(axHandles(1), 'XTickLabel', parameters.XTickLabels{2});
end

if(isempty(parameters.XTick{2}))
    set(axHandles(1), 'XColor', whiteColor);
end

if(~isempty(parameters.YRange{2}))
    set(axHandles(1), 'YLim', parameters.YRange{2});
else
    set(axHandles(1), 'YLim', parameters.YRange{1});
end

if(strcmp(parameters.YTick{2}, 'auto'))
    set(axHandles(1), 'YTickLabelMode', 'auto');
else
    set(axHandles(1), 'YTick', parameters.YTick{2});
    set(axHandles(1), 'YTickLabel', parameters.YTickLabels{2});
end

set(axHandles(1), 'TickDir', 'out');
set(axHandles(1), 'LineWidth', 2);

tempXLabel      = repmat({' '}, 1, length(get(axHandles(1), 'XTickLabel')));
tempYLabel      = repmat({' '}, 1, length(get(axHandles(1), 'YTickLabel')));

axHandles(2)    = axes('XAxisLocation', 'top', 'YAxisLocation', 'right', 'XLim', get(axHandles(1), 'XLim'), 'YLim', get(axHandles(1), 'YLim'));

set(axHandles(2), 'XTick', get(axHandles(1), 'XTick'), 'YTick', get(axHandles(1), 'YTick'));
set(axHandles(2), 'XTickLabel', tempXLabel, 'YTickLabel', tempYLabel);
set(axHandles(2), 'TickDir', 'out');
set(axHandles(2), 'YColor', whiteColor);
set(axHandles(2), 'LineWidth', 2);

axHandles(3)    = axes('XAxisLocation', 'bottom', 'YAxisLocation', 'left');

set(axHandles(3), 'XLim', parameters.XRange{1});

if(strcmp(parameters.XTick(1), 'auto'))
    set(axHandles(3), 'XTickLabelMode', 'auto');
else
    set(axHandles(3), 'XTick', parameters.XTick{1});
    set(axHandles(3), 'XTickLabel', parameters.XTickLabels{1});
end

set(axHandles(3), 'YLim', parameters.YRange{1});

if(strcmp(parameters.YTick(1), 'auto'))
    set(axHandles(3), 'YTickLabelMode', 'auto');
else
    set(axHandles(3), 'YTick', parameters.YTick{1});
    set(axHandles(3), 'YTickLabel', parameters.YTickLabels{1});
end

set(axHandles(3), 'TickDir', 'out');

tempXLabel      = repmat({' '}, 1, length(get(axHandles(3), 'XTickLabel')));
tempYLabel      = repmat({' '}, 1, length(get(axHandles(3), 'YTickLabel')));

axHandles(4)    = axes('XLim', get(axHandles(1), 'XLim'), 'YLim', get(axHandles(3), 'YLim'));
set(axHandles(4), 'XTick', get(axHandles(1), 'XTick'), 'YTick', get(axHandles(3), 'YTick'));
set(axHandles(4), 'XTickLabel', tempXLabel, 'YTickLabel', tempYLabel);
set(axHandles(4), 'TickDir', 'out');
set(axHandles(4), 'YColor', whiteColor);
set(axHandles(4), 'Color', greyColor);
set(axHandles(4), 'YGrid', 'on');
set(axHandles(4), 'GridLineStyle', '-');
set(axHandles(4), 'LineWidth', 2);

for(i = 1:length(parameters.XData))
    if(isempty(parameters.colorStyle{i}) & isempty(parameters.lineStyle{i}) & isempty(parameters.markerStyle{i}))
        tempPlotStyle   = getNextPlotStyle;

        %         while(any(strcmp(cell2mat(tempPlotStyle), parameters.plotStyle))
        %             tempPlotStyle   = getNextPlotStyle;
        %         end

        parameters.colorStyle{i}    = tempPlotStyle{1};
        parameters.lineStyle{i}     = '-';
        parameters.markerStyle{i}   = '';
    end

    plotString  = 'line(parameters.XData{i}, parameters.YData{i}, ';

    if(~isempty(parameters.colorStyle{i}))
        if(ischar(parameters.colorStyle{i}))
            plotString  = [plotString, sprintf('''Color'', ''%s''', parameters.colorStyle{i}) ', '];
        elseif(isnumeric(parameters.colorStyle{i}))
            plotString  = [plotString, sprintf('''Color'', [%f %f %f]', parameters.colorStyle{i}(1), parameters.colorStyle{i}(2), parameters.colorStyle{i}(3)) ', '];
        end
    end

    if(~isempty(parameters.lineStyle{i}))
        plotString  = [plotString, sprintf('''LineStyle'', ''%s''', parameters.lineStyle{i}) ', '];
    end

    if(~isempty(parameters.markerStyle{i}))
        plotString  = [plotString, sprintf('''Marker'', ''%s''', parameters.markerStyle{i}) ', '];
    end

    if(~isempty(parameters.addStyle{i}))
        plotString  = [plotString, parameters.addStyle{i}, ')'];
    else
        plotString  = [plotString(1:end-2), ')'];
    end

    try
        axes(axHandles(4));
        eval(plotString);
    catch
        error(sprintf('Error in plotting data set #%d', i));
    end
end

% set(get(axHandles(1), 'Title'), 'String', parameters.plotTitle);

set(get(axHandles(3), 'XLabel'), 'String', parameters.XLabels{1});
set(get(axHandles(3), 'YLabel'), 'String', parameters.YLabels{1});

%set(get(axHandles(1), 'XLabel'), 'String', parameters.XLabels{2});
%set(get(axHandles(1), 'YLabel'), 'String', parameters.YLabels{2});

set(axHandles(1), 'Position', get(axHandles(1), 'Position'));
set(axHandles(2), 'Position', get(axHandles(1), 'Position'));
set(axHandles(3), 'Position', get(axHandles(1), 'Position'));
set(axHandles(4), 'Position', get(axHandles(1), 'Position'));

if(parameters.legendFlag)
    legend(axHandles(4), parameters.dataLegend, 'Location', 'Best');
    legend(axHandles(4), 'boxoff');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function parameters = check_parameters(parameters);

defNumXTicks    = 11;
defNumYTicks    = 11;

if(~isstruct(parameters))
    error('parameters is of non-structure format');
end

if(~isfield(parameters, 'XData') | ~isfield(parameters, 'YData'))
    error('x and y data not specified properly')
else
    numXData    = length(parameters.XData);
    numYData    = length(parameters.YData);

    if(numXData ~= numYData)
        error('Number of x and y data entries do not agree');
    end
end

if(~isfield(parameters, 'colorStyle'))
    parameters.colorStyle   = cell(1, numXData);
elseif(length(parameters.colorStyle) < numXData)
    temp                                    = parameters.colorStyle;
    parameters.colorStyle                   = cell(1, numXData);
    parameters.colorStyle(1:length(temp))   = temp;
end

if(~isfield(parameters, 'lineStyle'))
    parameters.lineStyle    = cell(1, numXData);
elseif(length(parameters.lineStyle) < numXData)
    temp                                    = parameters.lineStyle;
    parameters.lineStyle                    = cell(1, numXData);
    parameters.lineStyle(1:length(temp))    = temp;
end

if(~isfield(parameters, 'markerStyle'))
    parameters.markerStyle  = cell(1, numXData);
elseif(length(parameters.markerStyle) < numXData)
    temp                                    = parameters.markerStyle;
    parameters.markerStyle                  = cell(1, numXData);
    parameters.markerStyle(1:length(temp))  = temp;
end


if(~isfield(parameters, 'addStyle'))
    parameters.addStyle = repmat({sprintf('''LineWidth'', 3, ''MarkerSize'', 4')}, 1, numXData);
elseif(length(parameters.addStyle) < numXData)
    temp                = parameters.addStyle;
    parameters.addStyle = [temp repmat({sprintf('''LineWidth'', 3, ''MarkerSize'', 4')}, 1, numXData-length(temp))];
end

if(~isfield(parameters, 'plotTitle'))
    parameters.plotTitle    = 'No Title Specified in Parameters';
end

if(~isfield(parameters, 'legendFlag'))
    parameters.legendFlag   = 0;
end
    
if(~isfield(parameters, 'dataLegend'))
    parameters.dataLegend   = repmat({'No Data Legend Specified in Parameters'}, 1, numXData);
elseif(length(parameters.dataLegend) < numXData)
    temp                    = parameters.dataLegend;
    parameters.dataLegend   = [temp repmat({'No Data Legend Specified in Parameters'}, 1, numXData-length(temp))];
end

if(~isfield(parameters, 'XRange'))
    minX    = NaN;
    maxX    = NaN;

    for(i = 1:numXData)
        minX    = min(min(parameters.XData{i}), minX);
        maxX    = max(max(parameters.XData{i}), maxX);
    end

    parameters.XRange{1}        = [minX, maxX];
    parameters.XTick{1}         = 'auto'; %linspace(minX, maxX, defNumXTicks);
    parameters.XTickLabels{1}   = parameters.XTick{1};
    parameters.XRange{2}        = [];
    parameters.XTick{2}         = [];
    parameters.XTickLabels{2}   = [];
    numXAxis                    = 1;
else
    numXAxis    = 0;
    
    for(i = 1:length(parameters.XRange))
        numXAxis    = numXAxis+(~isempty(parameters.XRange{i}));
    end
    
    numXAxis            = min(numXAxis, 2);
    parameters.XRange   = parameters.XRange(1:numXAxis);

    if(numXAxis == 1)
        parameters.XRange{2}    = [];
    end
end

if(~isfield(parameters, 'YRange'))
    minY    = NaN;
    maxY    = NaN;

    for(i = 1:numYData)
        minY    = min(min(parameters.YData{i}), minY);
        maxY    = max(max(parameters.YData{i}), maxY);
    end

    parameters.YRange{1}        = [minY, maxY];
    parameters.YTick{1}         = 'auto'; % linspace(minY, maxY, defNumYTicks);
    parameters.YTickLabels{1}   = parameters.YTick{1};
    parameters.YRange{2}        = [];
    parameters.YTick{2}         = [];
    parameters.YTickLabels{2}   = [];
    numYAxis                    = 1;
else
    numYAxis    = 0;
    
    for(i = 1:length(parameters.YRange))
        numYAxis    = numYAxis+(~isempty(parameters.YRange{i}));
    end
    
    numYAxis            = min(numYAxis, 2);
    parameters.YRange   = parameters.YRange(1:numYAxis);

    if(numYAxis == 1)
        parameters.YRange{2}    = [];
    end
end

if(~isfield(parameters, 'XTick'))
    for(i = 1:numXAxis)
        parameters.XTick{i}         = 'auto'; % linspace(parameters.XRange{i}(1), parameters.XRange{i}(2), defNumXTicks);
        parameters.XTickLabels{i}   = parameters.XTick{i};
    end

    if(numXAxis == 1)
        parameters.XTick{2}         = [];
        parameters.XTickLabels{2}   = [];
    end

else
    numXTick    = 0;
    
    for(i = 1:length(parameters.XTick))
        numXTick    = numXTick+(~isempty(parameters.XTick{i}));
    end
    
    numXTick    = min(numXTick, 2);

    if(numXTick ~= numXAxis)
        parameters.XTick    = parameters.XTick(1:min(numXAxis, numXTick));

        if(numXTick < numXAxis)
            for(i = numXTick+1:numXAxis)
                parameters.XTick{i}         = 'auto'; % linspace(parameters.XRange{i}(1), parameters.XRange{i}(2), defNumXTicks);
                parameters.XTickLabels{i}   = parameters.XTick{i};
            end
                
        elseif(numXTick > numXAxis)
            for(i = numXTick:-1:numXAxis+1)
                parameters.XTick{i}         = [];
                parameters.XTickLabels{i}   = [];
            end
        end
    end
end

if(~isfield(parameters, 'YTick'))
    for(i = 1:numYAxis)
        parameters.YTick{i}         = 'auto'; % linspace(parameters.YRange{i}(1), parameters.YRange{i}(2), defNumYTicks);
        parameters.YTickLabels{i}   = parameters.YTick{i};
    end

    if(numYAxis == 1)
        parameters.YTick{2}         = [];
        parameters.YTickLabels{2}   = [];
    end

else
    numYTick    = 0;
    
    for(i = 1:length(parameters.YTick))
        numYTick    = numYTick+(~isempty(parameters.YTick{i}));
    end
    
    numYTick    = min(numYTick, 2);

    if(numYTick ~= numYAxis)
        parameters.YTick    = parameters.YTick(1:min(numYAxis, numYTick));

        if(numYTick < numYAxis)
            for(i = numYTick+1:numYAxis)
                parameters.YTick{i}         = 'auto'; % linspace(parameters.YRange{i}(1), parameters.YRange{i}(2), defNumYTicks);
                parameters.YTickLabels{i}   = parameters.YTick{i};
            end
                
        elseif(numYTick > numYAxis)
            for(i = numYTick:-1:numYAxis+1)
                parameters.YTick{i}         = [];
                parameters.YTickLabels{i}   = [];
            end
        end
    end
end

if(~isfield(parameters, 'XTickLabels'))
    for(i = 1:numXAxis)
        parameters.XTickLabels{i}   = parameters.XTick{i};
    end

    if(numXAxis == 1)
        parameters.XTickLabels{2}   = [];
    end

else
    numXTickLabels  = 0;
    
    for(i = 1:length(parameters.XTickLabels))
        numXTickLabels  = numXTickLabels+(~isempty(parameters.XTickLabels{i}));
    end
    
    numXTickLabels  = min(numXTickLabels, 2);

    if(numXTickLabels ~= numXAxis)
        parameters.XTickLabels  = parameters.XTickLabels(1:min(numXAxis, numXTickLabels));

        if(numXTickLabels < numXAxis)
            for(i = 1:numXTickLabels+1:numXAxis)
                parameters.XTickLabels{i}   = parameters.XTick{i};
            end
                
        elseif(numXTickLabels > numXAxis)
            for(i = numXTickLabels:-1:numXAxis)
                parameters.XTickLabels{i}   = [];
            end
        end
    end
    
    for(i = 1:numXTickLabels)
        if(~strcmp(parameters.XTick{i}, 'auto') & strcmp(parameters.XTickLabels{i}, 'auto'))
            parameters.XTickLabels{i}   = parameters.XTick{i};
        end
    end
end

if(~isfield(parameters, 'YTickLabels'))
    for(i = 1:numYAxis)
        parameters.YTickLabels{i}   = parameters.YTick{i};
    end

    if(numYAxis == 1)
        parameters.YTickLabels{2}   = [];
    end

else
    numYTickLabels  = 0;
    
    for(i = 1:length(parameters.YTickLabels))
        numYTickLabels  = numYTickLabels+(~isempty(parameters.YTickLabels{i}));
    end
    
    numYTickLabels  = min(numYTickLabels, 2);

    if(numYTickLabels ~= numYAxis)
        parameters.YTickLabels  = parameters.YTickLabels(1:min(numYAxis, numYTickLabels));

        if(numYTickLabels < numYAxis)
            for(i = 1:numYTickLabels+1:numYAxis)
                parameters.YTickLabels{i}   = parameters.YTick{i};
            end
                
        elseif(numYTickLabels > numYAxis)
            for(i = numYTickLabels:-1:numYAxis)
                parameters.YTickLabels{i}   = [];
            end
        end
    end
    
    for(i = 1:numYTickLabels)
        if(~strcmp(parameters.YTick{i}, 'auto') & strcmp(parameters.YTickLabels{i}, 'auto'))
            parameters.YTickLabels{i}   = parameters.YTick{i};
        end
    end
end

if(~isfield(parameters, 'XLabels'))
    parameters.XLabels  = repmat({[]}, 1, 2);

    for(i = 1:numXAxis)
        if(~isempty(parameters.XTick{i}))
            parameters.XLabels{i}   = 'No X-Label Specified in Parameters';
        end
    end

else
    numXLabels  = 0;

    for(i = 1:length(parameters.XLabels))
        numXLabels  = numXLabels+ischar(parameters.XLabels{i});
    end
    
    numXLabels  = min(numXLabels, 2);
    
    if(numXLabels ~= numXAxis)
        parameters.XLabels  = [parameters.XLabels(1:min(numXAxis, numXLabels)), cell(1, 2-numXLabels)];

        if(numXLabels < numXAxis)
            for(i = numXLabels+1:numXAxis)
                parameters.XLabels{i}   = 'No X-Label Specified in Parameters';
            end

        elseif(numXLabels > numXAxis)
            for(i = numXLabels:-1:numXAxis)
                parameters.XLabels{i}   = [];
            end
        end
    end
end

if(~isfield(parameters, 'YLabels'))
    parameters.YLabels  = repmat({[]}, 1, 2);

    for(i = 1:numYAxis)
        if(~isempty(parameters.YTick{i}))
            parameters.YLabels{i}   = 'No Y-Label Specified in Parameters';
        end
    end

else
    numYLabels  = 0;

    for(i = 1:length(parameters.YLabels))
        numYLabels  = numYLabels+ischar(parameters.YLabels{i});
    end
    
    numYLabels  = min(numYLabels, 2);
    
    if(numYLabels ~= numYAxis)
        parameters.YLabels  = [parameters.YLabels(1:min(numYAxis, numYLabels)), cell(1, 2-numYLabels)];

        if(numYLabels < numYAxis)
            for(i = numYLabels+1:numYAxis)
                parameters.YLabels{i}   = 'No Y-Label Specified in Parameters';
            end

        elseif(numYLabels > numYAxis)
            for(i = numYLabels:-1:numYAxis)
                parameters.YLabels{i}   = [];
            end
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function plotStyle = getNextPlotStyle;

global colorStyle;
global lineStyle;
global markStyle;
global usedPlotStyles;

plotStyle   = [colorStyle(ceil(rand(1)*length(colorStyle))), lineStyle(ceil(rand(1)*length(lineStyle))), markStyle(ceil(rand(1)*length(markStyle)))];

while(any(strcmp(cell2mat(plotStyle), usedPlotStyles)))
    plotStyle   = {colorStyle(ceil(rand(1)*length(colorStyle))), lineStyle(ceil(rand(1)*length(lineStyle))), markStyle(ceil(rand(1)*length(markStyle)))};
end

usedPlotStyles  = [usedPlotStyles, {cell2mat(plotStyle)}];